/*
// $Id: //guest/julian_hyde/saffron/src/main/saffron/opt/Rule.java#1 $
// Saffron preprocessor and data engine
// Copyright (C) 2002 Julian Hyde <julian.hyde@mail.com>
// 
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Library General Public
// License as published by the Free Software Foundation; either
// version 2 of the License, or (at your option) any later version.
// 
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Library General Public License for more details.
// 
// You should have received a copy of the GNU Library General Public
// License along with this library; if not, write to the
// Free Software Foundation, Inc., 59 Temple Place - Suite 330,
// Boston, MA  02111-1307, USA.
// 
// See the COPYING file located in the top-level-directory of
// the archive of this library for complete text of license.
*/

package saffron.opt;

import openjava.mop.Environment;
import saffron.rel.Rel;

/**
 * A <code>Rule</code> transforms an expression into another.
 **/
abstract class Rule
{
    /**
     * Current planner; set by {@link VolcanoPlanner#VolcanoPlanner}.
     **/
    protected VolcanoPlanner planner;
    /** Root of operand tree. */
    Operand operand;
    /** Flattened list of operands. */
    Operand[] operands;

    Rule(Operand operand)
    {
        this.operand = operand;
    }

    /**
     * Return the current {@link VolcanoPlanner}.
     */
    VolcanoPlanner getPlanner()
    {
        return planner;
    }

    /**
     * Return the calling convention which the generated expression will
     * support.
     */
    public abstract int getConvention();

    /**
     * Apply this rule to an expression, to produce an expression whose
     * output matches <code>convention</code>.  If the rule is not
     * applicable, return null.
     *
     * @param rels the parameters to the operands of this rule; all
     *   <code>rels[i]</code> are expressions which match
     *   <code>operands[i]</code>; <code>rels[0]</code> is the root
     *   expression.
     */
    public abstract Rel apply(Environment env, Rel[] rels);

    /**
     * Return a description of this rule.
     */
    String getDescription()
    {
        return getClass().toString();
    }
}
